/***************************************************************************
                          controlpanel.cpp  -  description
                             -------------------
    begin                : Don Apr 17 2003
    copyright            : (C) 2003 by J. Kofler
    email                : kaffeine@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <klocale.h>
#include <kaccel.h>
#include <kglobal.h>
#include <kiconloader.h>

#include <qlayout.h>
#include <qnamespace.h>
#include <qtooltip.h>
#include <qwhatsthis.h>

#include "controlpanel.h"

ControlPanel::ControlPanel(QWidget *parent, const char *name ) : QWidget(parent,name), DCOPObject("PlaybackControl"),
 isPlaying(false), isPlayButton(true)
{

  setSizePolicy ( QSizePolicy (QSizePolicy::MinimumExpanding, QSizePolicy::Fixed) );
  
  
  QGridLayout* glayout= new QGridLayout(this, 2,8);
  glayout->setMargin(2);
  glayout->setSpacing(2);
  
  playPausePB= new KPushButton(this);
  playPixmap=KGlobal::iconLoader()->loadIcon("player_play", KIcon::Small);
  pausePixmap=KGlobal::iconLoader()->loadIcon("player_pause", KIcon::Small);
  playPausePB->setMaximumHeight(32);
  playPausePB->setPixmap(playPixmap);
  playPausePB->setFocusPolicy(QWidget::StrongFocus);
  glayout->addWidget(playPausePB, 1,1);

  stopPB=new KPushButton(this); 
  stopPB->setPixmap(KGlobal::iconLoader()->loadIcon("player_stop", KIcon::Small));
  glayout->addWidget(stopPB, 1,2);

  nextPB=new KPushButton(this);
  nextPB->setPixmap(KGlobal::iconLoader()->loadIcon("player_end", KIcon::Small));
  glayout->addWidget(nextPB, 1, 3);

  prevPB=new KPushButton(this);
  prevPB->setPixmap(KGlobal::iconLoader()->loadIcon("player_start", KIcon::Small));
  glayout->addWidget(prevPB, 1,0);

  playListPB = new KPushButton(this);
  playListPB->setPixmap(KGlobal::iconLoader()->loadIcon("contents", KIcon::Small));
  glayout->addWidget(playListPB, 1,4);
  connect(playListPB, SIGNAL(clicked()), this, SIGNAL(signalPlayList()));

  timeButton= new KPushButton(this);
  timeButton->setMaximumHeight(32);
  glayout->addWidget(timeButton, 0,6);
  connect(timeButton, SIGNAL(clicked()), this, SIGNAL(signalTogglePlayMode()));
  
  audioChannelCB= new KComboBox(this);
  audioChannelCB->insertItem("auto");
  audioChannelCB->setSizePolicy ( QSizePolicy (QSizePolicy::Minimum, QSizePolicy::Fixed) );
  glayout->addWidget(audioChannelCB,1, 5);
  connect(audioChannelCB, SIGNAL(activated(int)), this, SIGNAL(signalSetAudioChannel(int)));

  subtitleCB= new KComboBox(this);
  subtitleCB->insertItem("off");
  subtitleCB->setSizePolicy ( QSizePolicy (QSizePolicy::Minimum, QSizePolicy::Fixed) );
  glayout->addWidget(subtitleCB,1, 6);
  connect(subtitleCB, SIGNAL(activated(int)), this, SIGNAL(signalSetSubtitleChannel(int)));

  volSlider= new  QSlider(Qt::Vertical, this);
  volSlider->setTickmarks(QSlider::Left);
  volSlider->setRange(-100, 0);
  volSlider->setSteps(1, 10);
  glayout->addMultiCellWidget(volSlider, 0, 1, 7, 7);
  connect(volSlider, SIGNAL(valueChanged(int)), this, SIGNAL(signalVolumeChanged(int)));

  posSlider= new QSlider(Qt::Horizontal, this);
  posSlider->setRange(0,65535);
  posSlider->setSteps(100, 1000);
  glayout->addMultiCellWidget(posSlider, 0,0,0,5);
  connect(posSlider, SIGNAL(sliderMoved(int)), this, SIGNAL(signalChangePosition(int)));
  
  QToolTip::add(audioChannelCB, i18n("Audiochannel"));
  QWhatsThis::add(audioChannelCB, i18n("Here you can select the Audio-Channel, default is auto"));
  QToolTip::add(subtitleCB, i18n("Subtitles"));
  QWhatsThis::add(subtitleCB, i18n("Here you can select the Subtitle-Channel if any, and turn subtitles off (default)"));
  QToolTip::add(volSlider, i18n("Volume") + "    +/-");
  QToolTip::add(posSlider, i18n("Position") + "     " + i18n("Left") + "/" + i18n("Right"));
  QToolTip::add(timeButton,i18n("Change Counter-Mode"));
  QWhatsThis::add(timeButton, i18n("Click this button to change the counter-mode. Modes are: count forward, count backwards and percent of stream."));
  QToolTip::add(playPausePB, i18n("Play/Pause") + "    " + i18n("Space"));
  QToolTip::add(nextPB, i18n("Next Entry/Chapter") + "    " + i18n("PageUp"));
  QWhatsThis::add(nextPB, i18n("Plays next laylist entry or next chapter, if there are chapters"));
  QToolTip::add(prevPB, i18n("Previous Entry/Chapter") + "    " + i18n("PageDown"));
  QWhatsThis::add(prevPB, i18n("Plays previous playlist entry or previous chapter, if there are chapters"));
  QToolTip::add(playListPB, i18n("Playlist") + "    P");
  QToolTip::add(stopPB, i18n("Stop") + "    " + i18n("Backspace"));


  connect(playPausePB, SIGNAL(clicked()), this, SLOT(slotPlayPause()));
  connect(stopPB, SIGNAL(clicked()), this, SLOT(slotStop()));
  connect(prevPB, SIGNAL(clicked()), this, SLOT(slotPrev()));
  connect(nextPB, SIGNAL(clicked()), this, SLOT(slotNext()));

  ResetControlPanel();


// key accels:
  KAccel* accel = new KAccel(parentWidget());

  accel->insert( "VolUp", Qt::Key_Plus, volSlider, SLOT(subtractStep()) );
  accel->insert( "VolDown", Qt::Key_Minus, volSlider, SLOT(addStep()) );
  accel->insert( "PosUp", Qt::Key_Right, this, SLOT(slotPosPlus()) );
  accel->insert( "PosDown", Qt::Key_Left, this, SLOT(slotPosMinus()) );
}


ControlPanel::~ControlPanel()
{
}


void ControlPanel::slotPosPlus()
{
  posSlider->addPage();
  emit signalChangePosition(posSlider->value());
}


void ControlPanel::slotPosMinus()
{
  posSlider->subtractPage();
  emit signalChangePosition(posSlider->value());
}
 

void ControlPanel::slotPlayPause()
{
  if (!isPlaying)
  {
    emit signalPlay();
  }
  else
  {

    if (!isPlayButton)
    {
       isPlayButton=true;
       playPausePB->setPixmap(playPixmap);
       emit signalPauseOn();
    }
    else
    {
      isPlayButton=false;
      playPausePB->setPixmap(pausePixmap);
      emit signalPauseOff();
    }     
  }
}                          


void ControlPanel::slotStop()
{  
  if (isPlaying)
   emit signalStop();
}

void ControlPanel::slotNext()
{
  if (isPlaying)
   emit signalPlayNext();
}


void ControlPanel::slotPrev()
{
  if (isPlaying)
   emit signalPlayPrevious();
}

    
void ControlPanel::ResetControlPanel()
{
  isPlaying=false;
  isPlayButton=true;
  playPausePB->setPixmap(playPixmap);
  timeButton->setText(QString("0:00:00"));
  posSlider->setValue(0);
}   

void ControlPanel::setVolume(int vol)
{
  volSlider->setValue(-vol);
}  

void ControlPanel::SetPlaying()
{
  isPlaying=true;
  isPlayButton=false;
  playPausePB->setPixmap(pausePixmap);
}

bool ControlPanel::GetPlaying() const
{
  return isPlaying;
}  

void ControlPanel::slotNewChannels(const QStringList& audio, const QStringList& subtitles, int currentAudio, int currentSub)
{
  audioChannelCB->clear();
  audioChannelCB->insertItem(i18n("auto"));
  audioChannelCB->insertStringList(audio);
  audioChannelCB->setCurrentItem(currentAudio+1);

  subtitleCB->clear();
  subtitleCB->insertItem(i18n("off"));
  subtitleCB->insertStringList(subtitles);
  subtitleCB->setCurrentItem(currentSub+1);
}

void ControlPanel::slotNewPosition(int pos, const QString& time)
{
  
  posSlider->setValue(pos);
    
  timeButton->setText(time);
}












  


