/***************************************************************************
                          kaffeine.h  -  description
                             -------------------
    begin                : Don Apr 17 11:30:44 CEST 2003
    copyright            : (C) 2003 by J.Kofler
    email                : kaffeine@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KMP_H
#define KMP_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kapp.h>
#include <kstdaction.h>
#include <klocale.h>
#include <kaction.h>
#include <kpopupmenu.h>
#include <kmenubar.h>
#include <kaboutapplication.h>
#include <khelpmenu.h>
#include <kfiledialog.h>
#include <kbookmarkmanager.h>
#include <kbookmarkmenu.h>
#include <dcopobject.h>

#include <qwidget.h>
#include <qlayout.h>
#include <qsignalmapper.h>
#include <qtimer.h>

#include "splashwidget.h"
#include "controlpanel.h"
#include "inforow.h"
#include "playlist.h"
#include "configwidget.h"
#include "xineconfig.h"
#include "videosettings.h"
#include "videowindow.h"



/** KMP is the base class of the project */
class KMP : public QWidget, virtual public KBookmarkOwner, public DCOPObject
{
  Q_OBJECT
  K_DCOP
  public:
    /** construtor */
    KMP(QWidget* parent=0, const char *name=0);
    /** destructor */
    ~KMP();

    void Finalize(); /* ...the gui */
    
   /* parse command line options */
    void CmdArgs(const KURL::List& files, const QString device, bool playAtStartup, bool startFullscreen, bool minimalMode);
  /* execute a command string like play, pause, next, etc; */  
    void ExecuteRemoteCommand(const QString&);  
    void LoadPlaylist();

    VideoWindow* videoWin;

   /* overloaded bookmarkowner methods */
    virtual void openBookmarkURL(const QString&);
    virtual QString currentTitle() const;
    virtual QString currentURL() const;

  /*** DCOP Interface ***/  
  k_dcop:
    void playURL(QString url) { slotPlayRecent(KURL(url)); }
    void play() { ExecuteRemoteCommand("play"); }
    void pause() { ExecuteRemoteCommand("pause"); }
    void next() { ExecuteRemoteCommand("next"); }
    void previous() { ExecuteRemoteCommand("prev"); }
    void stop() { ExecuteRemoteCommand("stop"); }
    QString getTitle() { if (cTitle.isNull()) return QString("Kaffeine");
                         return cTitle;
                       }  
    void playlist() { slotShowPlayList(); }
    void quit() { slotQuit(); }
    void random() { playRandomPlay->setChecked(!playRandomPlay->isChecked());
                    slotRandomPlay();
                  }
    void fullscreen() { viewFullscreen->setChecked(!viewFullscreen->isChecked());
                        slotToggleFullscreen();
                      } 
    
  private slots:
    void slotQuit();
    void slotOpenFile();
    void slotOpenUrl();
    void slotPlay();
    void slotEndOfPlayback();
    void slotPlayNext();
    void slotPlayPrev();
    void slotPlayRecent(const KURL&); /* choosen file from recent menu */
    void slotStop();
    void slotToggleFullscreen();
    void slotToggleMinimalMode();
    void slotPlayAutoplayPlugin(const QString&); /* e.g. dvd:// */
    void slotShowPlayList(); /* toggel visibility of playlist window */
    void slotShowSetup();
    void slotHasChapters(bool);  /* set if stream has chapters or not */
    void slotEndlessMode();
    void slotRandomPlay();
    void slotGenConfig(bool loadPl);
    void slotConfigXine();
    void slotVideoSettings();
    void slotScreenshot();
    void slotToggleTv();  /* not implemeted*/
    void slotAddAutoplayUrls(const QString& plugin);

   //for context menu 
    void slotShowContextMenu(const QPoint&);
    void slotContextMenuVolume(int);
    void slotContextMenuAudioChannel(int);
    void slotContextMenuSubtitleChannel(int);
    
  protected:

  virtual void closeEvent(QCloseEvent*);
  virtual void moveEvent(QMoveEvent*);
  virtual void dropEvent(QDropEvent*);
  virtual void dragEnterEvent(QDragEnterEvent*);
    
  private:

  PlayList* playList;
  ControlPanel* cPanel;
  TextRow* tRow;
  SplashWidget* osDisplay;
  ConfigWidget* configWin;
  VideoSettings* videoSettings;

  QString currentAutoPlayMrl;

  bool playSingleUrl;
  QString cUrl;
  QString cTitle;
  
  void InitActions();
  void InitMenu();
  void InitContextMenu();
  void InitConnections();
  void ReadConfig();
  void SaveConfig();
  
  bool hasChapters;
  bool loadPlaylistAtStartup;
  
  KConfig* config;

  KMenuBar* menuBar;
  KPopupMenu* fileMenu;
  KPopupMenu* playMenu;
  KPopupMenu* viewMenu;
  KPopupMenu* aspectMenu;
  KPopupMenu* configMenu;
  KPopupMenu* helpMenu;

 //context menu
  KPopupMenu* contextMenu;
  KPopupMenu* controlMenu;
  KPopupMenu* audioChannelMenu;
  KPopupMenu* subtitleChannelMenu;
  KPopupMenu* volumeMenu;

  KPopupMenu* bookmarkMenu;
  KBookmarkMenu* bmenu;
  KBookmarkManager* bookmarkManager;

  KActionCollection* actionCollection;
  KAction* fileOpen;
  KAction* fileOpenUrl;
  KAction* fileSaveScreenshot;
  KAction* fileEject;
  KAction* fileQuit;
  KRecentFilesAction* playRecent;
  KToggleAction* playEndlessPlay;
  KToggleAction* playRandomPlay;
  KToggleAction* viewFullscreen;
  KToggleAction* viewMinimalMode;

  KAction* aspectRatioAuto;
  KAction* aspectRatio4_3;
  KAction* aspectRatio16_9;
  KAction* aspectRatioSquare;
  
  KToggleAction* viewDeinterlace;
  KAction* viewZoomIn;
  KAction* viewZoomOut;
  KAction* viewZoomOff;
  KToggleAction* viewToggleTv;
  KAction* configVideoControl;
  KAction* configXine;
  KAction* configKMP;

  KAction* autoPlayPlugin;

  //actions for context menu
  
  KAction* controlPlayPause;
  KAction* controlNext;
  KAction* controlPrevious;
  KAction* controlStop;
  KAction* controlShowHidePlaylist;

  QTimer playDelayedTimer;
      
};


#endif
