/***************************************************************************
                          kaffeine_part.cpp  -  description
                             -------------------
    begin                : Don Apr 17 11:30:44 CEST 2003
    copyright            : (C) 2003 by J. Kofler
    email                : kaffeine@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/




#include <kiconloader.h>
#include <klocale.h>
#include <kaction.h>
#include <kglobalsettings.h>
#include <kdebug.h>

#include <qiconset.h>
#include <qtimer.h>
#include <qgrid.h>

#include <kparts/partmanager.h>

#include "kaffeine_part.h"


extern "C"
{
  /**
   * This function is the 'main' function of this part.  It takes
   * the form 'void *init_lib<library name>()'.  It always returns a
   * new factory object
   */
  void *init_kaffeine_part()
  {
    return new KaffeineFactory;
  }
};


KInstance *KaffeineFactory::s_instance = 0L;


static const char *description =
 I18N_NOOP("A xine based media player. Take a look at Kaffeine Handbook for more information.");


KaffeineFactory::KaffeineFactory()
{
  s_instance=0L;
}


KaffeineFactory::~KaffeineFactory()
{
  if (s_instance)
  {
    delete s_instance->aboutData();
    delete s_instance;
  }

  s_instance = 0;
}


KParts::Part *KaffeineFactory::createPartObject(QWidget *parentWidget, const char *widgetName,
                                            QObject *parent, const char *name, const char*,
    const QStringList& )
{
  KParts::Part *obj = new KaffeinePart(parentWidget, widgetName, parent, name);
  return obj;
}


KAboutData *KaffeineFactory::aboutData()
{
    KAboutData* aboutData = new KAboutData( "kaffeine", I18N_NOOP("KAFFEINE"),
    VERSION, description, KAboutData::License_GPL,
    "(c) 2003, Jürgen Kofler.", 0, "http://kaffeine.sourceforge.net", "kaffeine@gmx.net");
    aboutData->addAuthor("Jürgen Kofler.",0, "kaffeine@gmx.net");

    return aboutData;
}

KInstance *KaffeineFactory::instance()
{
 
  if (!s_instance)
    s_instance = new KInstance( aboutData() );
    
  return s_instance;
}


KaffeinePart::KaffeinePart(QWidget *parentWidget, const char* /*widgetName*/,
                     QObject *parent, const char *name)
: KParts::ReadOnlyPart(parent, name), videoWin(NULL), infoRow(NULL), playList(NULL)
{
  KInstance* instance = KaffeineFactory::instance();
  setInstance(instance);

  QWidget* mainFrame = new QWidget(parentWidget, "kaffeinemain");
  QVBoxLayout* boxLayout = new QVBoxLayout(mainFrame);
  QWidget* panel = new QWidget(mainFrame, "panel");
  panel->setMaximumHeight(32);
  QHBoxLayout* panelLayout = new QHBoxLayout(panel);
  panelLayout->setSpacing(5);
  panelLayout->setMargin(2);
  
  videoWin = new VideoWindow(mainFrame, "videowin");
  videoWin->setUpdatesEnabled(false); //prevent qt overwrites videowin black
  infoRow = new TextRow(mainFrame, "inforow");
  connect(videoWin, SIGNAL(signalNewInfo(const QString&)), infoRow, SLOT(slotChangeText(const QString&)));

  KPushButton* previousButton = new KPushButton(panel);
  previousButton->setPixmap(KGlobal::iconLoader()->loadIcon("player_start", KIcon::Small));
  connect(previousButton, SIGNAL(clicked()), this, SLOT(slotPrevious()));

  KPushButton* playButton = new KPushButton(panel);
  playButton->setPixmap(KGlobal::iconLoader()->loadIcon("player_play", KIcon::Small));
  connect(playButton, SIGNAL(clicked()), this, SLOT(slotPlay()));
  
  KPushButton* stopButton = new KPushButton(panel);
  stopButton->setPixmap(KGlobal::iconLoader()->loadIcon("player_stop", KIcon::Small));
  connect(stopButton, SIGNAL(clicked()), this, SLOT(slotStop()));
  
  KPushButton* nextButton = new KPushButton(panel);
  nextButton->setPixmap(KGlobal::iconLoader()->loadIcon("player_end", KIcon::Small));
  connect(nextButton, SIGNAL(clicked()), this, SLOT(slotNext()));

  posSlider= new QSlider(Qt::Horizontal, panel);
  posSlider->setRange(0,65535);
  posSlider->setSteps(100, 1000);
  connect(posSlider, SIGNAL(sliderMoved(int)), videoWin, SLOT(slotChangePosition(int)));
  
  timeButton= new KPushButton(panel);
  connect(timeButton, SIGNAL(clicked()), videoWin, SLOT(slotTogglePlayMode()));
  connect(videoWin, SIGNAL(signalNewPosition(int, const QString&)), this, SLOT(slotSetPosition(int, const QString&)));

  KPushButton* logoButton = new KPushButton(panel);
  logoButton->setPixmap(KGlobal::iconLoader()->loadIcon("kaffeine", KIcon::Small));
  connect(logoButton, SIGNAL(clicked()), this, SLOT(slotGoHome()));
  
  panelLayout->addWidget(previousButton);
  panelLayout->addWidget(playButton);
  panelLayout->addWidget(stopButton);
  panelLayout->addWidget(nextButton);
  panelLayout->addWidget(posSlider);
  panelLayout->addWidget(timeButton);
  panelLayout->addWidget(logoButton);
  
  boxLayout->addWidget(videoWin);
  boxLayout->addWidget(infoRow);
  boxLayout->addWidget(panel);

  mainFrame->show();
  mainFrame->setFocusPolicy(QWidget::ClickFocus);
  setWidget(mainFrame);

  //load configuration 

  KConfig* config = instance->config();
  config->setGroup("General Options");
  
  QString metaStr = config->readEntry("Meta String", QString("%1 - %2 (%3)"));
  videoWin->slotSetConfig(metaStr, 1000);

  QFont f = KGlobalSettings::generalFont();
  QFont infoFont = config->readFontEntry("Textrow Font", &f);
  QColor c = KGlobalSettings::baseColor();
  QColor infoColor = config->readColorEntry("Textrow Color", &c);
  infoRow->slotSetConfig(&infoFont, &infoColor);
  
  //***********************
  
  browserExtension = new KaffeineBrowserExtension(this);

}


KaffeinePart::~KaffeinePart()
{

  /* FIXME: crash */  

//  kdDebug(555) << "kaffeine part deconstructor\n";

  if (videoWin->xineRunning)
     videoWin->ExitXine();
}


bool KaffeinePart::openURL(const KURL& url)
{
  if (url.isMalformed()) return false;
  m_url = url;
  m_bTemp = false;

  QTimer::singleShot(200, this, SLOT(slotPlay()));

  return true;
}


bool KaffeinePart::closeURL()
{
  if (playList)
   playList->slotClearList();
  return true; 
}   
  

void KaffeinePart::slotPlay()
{
  if (!videoWin->xineRunning)
  {
    if (!videoWin->InitXine())
    {
      emit canceled(i18n("Can't init Xine Engine!"));
    }
    playList = new PlayList(videoWin->GetMimeTypes(), 0, "playlist");
    playList->Add(m_url, 0);
  }

  QListViewItem* item = playList->GetCurrent();

  if (!item) return;

  videoWin->PlayMRL(item->text(4), item->text(1), false);
}


void KaffeinePart::slotStop()
{
 if (videoWin->xineRunning)
   videoWin->slotStopPlayback();

 slotSetPosition(0, "0:00:00");
 infoRow->slotChangeText(i18n("Ready."));  
}


void KaffeinePart::slotNext()
{
  QListViewItem* item = playList->GetNext();

  if (!item) return;

  videoWin->PlayMRL(item->text(4), item->text(1), false);
}


void KaffeinePart::slotPrevious()
{
  QListViewItem* item = playList->GetPrevious();

  if (!item) return;

  videoWin->PlayMRL(item->text(4), item->text(1), false);
}


void KaffeinePart::slotGoHome()
{
  browserExtension->createNewWindow("http://kaffeine.sourceforge.net");
}


void KaffeinePart::slotSetPosition(int pos, const QString& time)
{
  posSlider->setValue(pos);
  timeButton->setText(time);
}  


KaffeineBrowserExtension::KaffeineBrowserExtension(KaffeinePart *parent)
    : KParts::BrowserExtension(parent, "KaffeineBrowserExtension")
{
}


KaffeineBrowserExtension::~KaffeineBrowserExtension()
{
}


