/***************************************************************************
                          playlist.h  -  description
                             -------------------
    begin                : Die Apr 22 2003
    copyright            : (C) 2003 by J. Kofler
    email                : kaffeine@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PLAYLIST_H
#define PLAYLIST_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kaction.h>
#include <kurl.h>
#include <kpopupmenu.h>
#include <kmenubar.h>
#include <kfileitem.h>
#include <kdirlister.h>
#include <ktoolbar.h>
#include <klistview.h>

#include <qlayout.h>
#include <qptrlist.h>
#include <qstrlist.h>
#include <qpixmap.h>

#include "urllistview.h"



   

class PlayList : public QWidget  {
   Q_OBJECT

friend class MyXMLParser;
friend class NoatunXMLParser;

   
public: 
	PlayList(const QString& mimes, QWidget *parent=0, const char *name=0);
	~PlayList();

 QListViewItem* GetCurrent();  /* get current playlist entry */
 QListViewItem* GetNext();
 QListViewItem* GetPrevious();


/* insert a KURL(list) after a specific item */ 
 void Add(const KURL& mrl, QListViewItem* after);
 void Add(const KURL::List& mrls, QListViewItem* after);
/* add autoplay-plugin urls given from xine */ 
 void AddAutoplayUrls(const QStringList& mrls, QListViewItem* after);
 
 void SetEndless(bool e);
 void SetRandomPlay(bool r);
 void SetLastPlaylist(const QString& pl);
 void SetAutoplayList(const QStringList& autoplay) { autoplayList = autoplay; }

 const QString& GetLastPlaylist() const;
 void ReloadLastPlaylist();
 void SaveCurrentPlaylist();

 signals:
  void signalPlayDirect();
  void signalAutoplayPlugin(const QString& id);
    
 public slots:
  void slotAddMrl(const QString&);  /* add a mrl sent by xine (as a child item) */
  void slotGetLengthInfo(const QString& time);
  void slotGetMetaInfo(const QString& metaInfo); /* e.g. mp3's */
  void slotGetStreamInfo(const QString& streamInfo);
  void slotClearList();  /* clear the whole list */
  void slotSetMetaInfoString(const QString&);

 private slots:
  void slotPlayDirect(QListViewItem* item);  /* doubleclick */
  
  void slotMove(QPtrList<QListViewItem>&, QPtrList<QListViewItem>&, QPtrList<QListViewItem>&);
  void slotDropEvent(QDropEvent*, QListViewItem*);
  void slotAddFiles();
  void slotAddUrl();
  void slotAddDir();
  void slotGetDirItems(const KFileItemList& items);
 
  void slotPaste();
  void slotRemoveSelected();
  void slotOpenPlaylist();
  void slotSavePlaylist();
  void slotFindText(const QString&);

  void slotSortByTitle();
  void slotSortByType();

  void slotExportHtml();
  void slotImportPls();
  void slotImportNoatun();
  void slotImportM3U();
  void slotImportAsx();

  void slotPrint();
  
  
 private:

  void InitActions();
  void InitMenus();

  QListViewItem* CreateItem(QListViewItem* after, QString url, QString mime = QString::null,
                            QString title = QString::null, QListViewItem* parent = NULL,
                            QString length = QString::null, QString info = QString::null);

  bool LoadPlaylist(const QString&, QListViewItem*);
  bool LoadNoatunPlaylist(const QString&, QListViewItem*);
  bool LoadM3UPlaylist(const QString&, QListViewItem*);
  bool LoadPlsPlaylist(const QString&, QListViewItem*);
  bool LoadAsxPlaylist(const QString&, QListViewItem*);

  bool SavePlaylist(const QString&);
  
  void SetCurrentEntry(QListViewItem*);

  QString lastPlaylist;
 
  UrlListView* list;
  QListViewItem* currentEntry;
  QListViewItem* currentChild;
  QString mimeTypes;

  QString metaInfoString;
  QStringList autoplayList;

  void CreateRandomList();
  QPtrList<QListViewItem> randomList;
  int currentRandomListEntry;

  QPixmap isCurrentEntry;
  QPixmap infoPixmap;
  QPixmap cdPixmap;
  QPixmap bookmarkPixmap;
  
  bool endless;
  bool random;

  QVBoxLayout* boxLayout;

  KPopupMenu* fileMenu;
  KPopupMenu* addMenu;
  KPopupMenu* editMenu;
  KPopupMenu* importMenu;
  KPopupMenu* exportMenu;
  KPopupMenu* sortMenu;

  KMenuBar* menuBar;
  KToolBar* toolBar;
  
  KActionCollection* actionCollection;

  KAction* fileSavePL;
  KAction* fileOpenPL;
  KAction* fileExportHtml;
  KAction* fileImportPls;
  KAction* fileImportNoatun;
  KAction* fileImportM3U;
  KAction* fileImportAsx;
  KAction* filePrint;
  KAction* fileClose;

  KAction* addFiles;
  KAction* addUrl;
  KAction* addDir;
  KAction* addPaste;
  KAction* editRemove;
  KAction* editClear;

  KAction* sortByTitle;
  KAction* sortByType;

  KDirLister* dirLister;
    
};

#endif
