/***************************************************************************
                          urllistview.cpp  -  description
                             -------------------
    begin                : Mon Apr 28 2003
    copyright            : (C) 2003 by J. Kofler
    email                : kaffeine@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

 
#include <kglobal.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kdebug.h>

#include "urllistview.h"


UrlListView::UrlListView(QWidget *parent, const char *name ) : KListView(parent,name), listCleared(true), itemOfContextMenu(0)
{
  contextMenu = new KPopupMenu(this);
  contextMenu->insertItem(KGlobal::iconLoader()->loadIconSet("player_play", KIcon::Small), i18n("&Play"), this, SLOT(slotPlayItem()));
  contextMenu->insertItem(KGlobal::iconLoader()->loadIconSet("edit", KIcon::Small), i18n("&Edit title"), this, SLOT(slotEditTitle()));
  contextMenu->insertItem(KGlobal::iconLoader()->loadIconSet("editdelete", KIcon::Small), i18n("&Delete entry"), this, SLOT(slotDeleteItem()));
  contextMenu->insertSeparator();
  contextMenu->insertItem(KGlobal::iconLoader()->loadIconSet("info", KIcon::Small), i18n("&Info"), this, SLOT(slotShowInfo()));
 
  connect(this, SIGNAL(contextMenuRequested(QListViewItem*, const QPoint&, int)),
          this, SLOT(slotShowContextMenu(QListViewItem*, const QPoint&, int)));
}


UrlListView::~UrlListView()
{
}

bool UrlListView::acceptDrag(QDropEvent* event) const
{
 return QUriDrag::canDecode(event) || QTextDrag::canDecode(event) || KListView::acceptDrag(event);
}

void UrlListView::resizeEvent(QResizeEvent* rev)
{

  setColumnWidth(0, 18);
  setColumnWidth(2, 50);
  setColumnWidth(3, 18);
  setColumnWidth(4, 18);
  setColumnWidth(1, contentsRect().width()-120);

  KListView::resizeEvent(rev);
}

void UrlListView::clear()
{
  listCleared = true;
 // kdDebug(555) << "Playlist cleared\n";
  
  KListView::clear();
}

void UrlListView::SetCleared(bool cl)
{
  listCleared = cl;
}

bool UrlListView::GetCleared() const
{
  return listCleared;
}

/********** context menu **********/  


void UrlListView::slotShowContextMenu(QListViewItem* item, const QPoint& pos, int)
{
  itemOfContextMenu = item;
  contextMenu->popup(pos);
}

void UrlListView::slotDeleteItem()
{
  clearSelection();
  itemOfContextMenu->setSelected(true);
  emit signalRemoveSelected();
}  


void UrlListView::slotPlayItem()
{
  emit signalPlayItem(itemOfContextMenu);
}
  

void UrlListView::slotEditTitle()
{
  itemOfContextMenu->setRenameEnabled(1, true);
  itemOfContextMenu->startRename(1);
  itemOfContextMenu->setRenameEnabled(1, false);
}
  

void UrlListView::slotShowInfo()
{
  QString num;
  num = num.setNum(childCount());
  QString info = itemOfContextMenu->text(1);
  info.append("\n");
  info = info + i18n("URL") + ": " + itemOfContextMenu->text(4) + "\n";
  info = info + i18n("Length") + ": " + itemOfContextMenu->text(2) + "\n";
  info = info + i18n("Stream") + ": " + itemOfContextMenu->text(3) + "\n";
  info = info + i18n("TOTAL: %1 entries").arg(num);
  
  KMessageBox::information(this, info);
}  


                
  

          

