/***************************************************************************
                          videowindow.h  -  description
                             -------------------
    begin                : Fre Apr 18 2003
    copyright            : (C) 2003 by J. Kofler
    email                : kaffeine@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef VIDEOWINDOW_H
#define VIDEOWINDOW_H



#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


#include <qwidget.h>
#include <qfile.h>
#include <qstringlist.h>
#include <qtimer.h>

#include <xine.h>

#include <pthread.h>

#include <X11/Xlib.h>
#include <X11/extensions/XShm.h>


class VideoWindow : public QWidget
{

 Q_OBJECT

friend class EventThread;
   
public: 
  VideoWindow(QWidget *parent=0, const char *name=0);
 ~VideoWindow();

  bool InitXine();
  void ExitXine();

  void GetAutoplayPlugins(QStringList&) const;
  bool GetAutoplayPluginMrl(const QString&, QStringList&);


/*******************/ 

  void ReDraw() const;  /* refresh, send an expose event to the xine display */

  bool PlayMRL(const QString& mrl, const QString& title, bool returnInfo);
  void PlayLOGO();
  bool xineRunning;  /* is xine engine running */

  int  GetVolume() const;
  void PlayNextChapter() const;
  void PlayPreviousChapter() const;

  QString GetMimeTypes() const;  /* get mime types xine can play */
  
  QString GetMetaString() const;
  int GetScreensaverTimeout() const; /* returns the timeout value in ms */
  
  void GetVideoSettings(int& hue, int& sat, int& contrast, int& bright, int& audioAmp,
                        int& avOffset, int& spuOffset) const;

 /* take a screenshot, will be stored in rgb32BitData */                       
  void GetScreenshot(uchar*& rgb32BitData, int& videoWidth, int& videoHeight, double& scaleFactor) const;

 /* get the xine engine, needed by the xine configuration dialog */
  const xine_t* const GetXineEngine() const;

  void StartMouseHideTimer();    /* for fullscreen */
  void StopMouseHideTimer();

  void SetDevice(const QString& device);

signals:
  void signalNewInfo(const QString&);  
  void signalHasChapters(bool);
  void signalPlaybackFinished();
  void signalNewMrlReference(const QString&);
  void signalNewChannels(const QStringList&, const QStringList&, int, int);
  void signalNewPosition(int, const QString&);
  void signalLengthInfo(const QString&);   /* length info now available */
  
  void signalMetaInfo(const QString&);
  void signalStreamInfo(const QString&);

  void signalShowOSD(const QString&);  /* show the osd title */

  void signalShowContextMenu(const QPoint&);  /* right-click on video window */
  
public slots:
  void slotSetVolume(int);
  void slotStopPlayback();
  void slotSpeedPause();
  void slotSpeedNormal();
  void slotSetAudioChannel(int);
  void slotSetSubtitleChannel(int);
  void slotChangePosition(int);
  void slotTogglePlayMode();
  void slotEject();
  void slotToggleDeinterlace();
  void slotAspectRatioAuto();
  void slotAspectRatio4_3();
  void slotAspectRatio16_9();
  void slotAspectRatioSquare();
  void slotZoomIn();
  void slotZoomOut();
  void slotZoomOff();
  void slotSetConfig(const QString& mstr, int ssTimeout);


  void slotSetHue(int hue);
  void slotSetSaturation(int sat);
  void slotSetContrast(int contrast);
  void slotSetBrightness(int bright);
  void slotSetAudioAmp(int amp);
  void slotSetAVOffset(int av);
  void slotSetSpuOffset(int spu);

private slots:
  void slotGetPosition();
  void slotEmitLengthInfo();
  void slotHideMouse();
  void slotFakeKeyEvent();

protected:
 virtual void mouseMoveEvent(QMouseEvent*);
 virtual void mousePressEvent(QMouseEvent*);
 

private:

 /********* callbacks and threads ************/
  static void DestSizeCallback(void* p, int video_width, int video_height, double video_aspect,
                       int* dest_width, int* dest_height, double* dest_aspect);

  static void FrameOutputCallback(void* p, int video_width, int video_height, double video_aspect,
                          int* dest_x, int* dest_y, int* dest_width, int* dest_height,
                          double* dest_aspect, int* win_x, int* win_y);

  static void VisualizationChangedCallback(void* p, xine_cfg_entry_t* entry);
  static void LogoPathChangedCallback(void* p, xine_cfg_entry_t* entry);

  static void XineEventListener(void* p, const xine_event_t*);

  static void* EventLoop(void* p);   /* event thread */

 //*****************
 
 void LockXineDisplay() const;
 void UnlockXineDisplay() const;

 enum PLAY_MODE { NORMAL_PLAY, REPEAT_PLAY, PERCENT_PLAY};
 PLAY_MODE playMode;

 QString GetTimeString(int msec);

 pthread_t eventThread;

 Window xineWindow;
 Display* xineDisplay;
 int xineScreen;

 xine_t* xineEngine;
 xine_audio_port_t* audioDriver;
 xine_video_port_t* videoDriver;
 xine_stream_t* xineStream;
 xine_event_queue_t* eventQueue;
 xine_post_t* visualPlugin;

 xine_post_t* xinePost;
 xine_post_out_t* postAudioSource;
 xine_post_in_t* postInput;
 
 double displayRatio;
 int shmEventId;

 int screensaverTimeout;
 int osdTimeout;
 int currentZoom;

 QTimer posTimer;
 QTimer lengthInfoTimer;
 QTimer screensaverTimer;
 QTimer mouseHideTimer;
 
 QString configFile;
 QString trackInfoRow;
 QString metaString;
 QString logoPath;

 QStringList audioCh;
 QStringList subCh;
 
 QString audioDriverName;
 QString videoDriverName;
 QString visualPluginName;

 QStringList xineErrors;

 int haveXTest, xTestKeycode;

 Window xComWindow;
 Atom atomQuit;
   
};

#endif
