/***************************************************************************
                          xineconfig.cpp  -  description
                             -------------------
    begin                : Son Mai 4 2003
    copyright            : (C) 2003 by J. Kofler
    email                : kaffeine@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <klocale.h>
#include <kdebug.h>

#include <qlabel.h>
#include <qscrollview.h>

#include "xineconfig.h"


XineConfigEntry::XineConfigEntry(QWidget* parent, QGridLayout* grid, int row, xine_cfg_entry_t* entry)
{

  key = QString(entry->key);
  valueChanged = false;
  numValue = entry->num_value;
  stringValue = entry->str_value;

  

  switch (entry->type)
  {
    case XINE_CONFIG_TYPE_UNKNOWN: break;
    case XINE_CONFIG_TYPE_STRING:
    {
      stringEdit = new KLineEdit(entry->str_value, parent);
      grid->addWidget(stringEdit, row ,0);
      connect(stringEdit, SIGNAL(textChanged(const QString&)), this, SLOT(slotStringChanged(const QString&)));
      break;
    }
    case XINE_CONFIG_TYPE_ENUM:
    {
      enumEdit = new KComboBox(parent);
      int i=0;
      while (entry->enum_values[i])
      {
        enumEdit->insertItem(entry->enum_values[i]);
        i++;
      }
      enumEdit->setCurrentItem(entry->num_value);
      grid->addWidget(enumEdit, row, 0);
      connect(enumEdit, SIGNAL(activated(int)), this, SLOT(slotNumChanged(int)));
      break;
    }
    case XINE_CONFIG_TYPE_NUM:
    {
      numEdit = new QSpinBox(parent);
      numEdit->setValue(entry->num_value);
      grid->addWidget(numEdit, row, 0);
      connect(numEdit, SIGNAL(valueChanged(int)), this, SLOT(slotNumChanged(int)));
      break;
    }
    case XINE_CONFIG_TYPE_RANGE:
    {
      numEdit = new QSpinBox(parent);
      numEdit->setValue(entry->num_value);
      numEdit->setRange(entry->range_min, entry->range_max);
      grid->addWidget(numEdit, row, 0);
      connect(numEdit, SIGNAL(valueChanged(int)), this, SLOT(slotNumChanged(int)));
      break;
    }       
    case XINE_CONFIG_TYPE_BOOL:
    {
      boolEdit = new QCheckBox(parent);
      boolEdit->setChecked(entry->num_value);
      grid->addWidget(boolEdit, row, 0);
      connect(boolEdit, SIGNAL(toggled(bool)), this, SLOT(slotBoolChanged(bool)));
      break;
    }        
  }

  QLabel* description = new QLabel(entry->description, parent);
  grid->addWidget(description, row, 1);
}


XineConfigEntry::~XineConfigEntry() {}

void XineConfigEntry::slotNumChanged(int val)
{
  
    numValue = val;
    valueChanged = true;
    
}

void XineConfigEntry::slotBoolChanged(bool val) 
{
 
   numValue = val;
   valueChanged = true;
}

void XineConfigEntry::slotStringChanged(const QString& val)
{

   stringValue = val;
   valueChanged = true;
}


bool XineConfigEntry::ValueChanged() const
{
  return valueChanged;
}
  
const QString& XineConfigEntry::GetKey() const
{
  return key;
}
  
int XineConfigEntry::GetNumValue() const
{
  return numValue;
}
  
const QString& XineConfigEntry::GetStringValue() const
{
  return stringValue;
}  

//*****************************************

XineConfig::XineConfig(const xine_t* const xine) :
       KDialogBase(KDialogBase::Tabbed, QString(i18n("Xine Config")), KDialogBase::Ok|KDialogBase::Apply|KDialogBase::Cancel, KDialogBase::Cancel)

{
  setInitialSize(QSize(600,500), true);
  
  xineEngine = (xine_t*)xine;
 
  xine_cfg_entry_t* ent;
  int row = 0;

  QString currentPage("");
  
  QString pageName;
  
  QScrollView* sv = NULL;
  QWidget* parent = NULL;
  QGridLayout* grid = NULL;


  ent = new xine_cfg_entry_t; 
  xine_config_get_first_entry(xineEngine, ent);

  do
  {
   if (ent->exp_level <=20)
    {
    
      pageName = QString(ent->key);
      pageName = pageName.left(pageName.find("."));
      
      
      if (pageName == currentPage)
      {
        row++;  
      }
      else
      {
      
       if (currentPage == "gui")
       {
         QLabel* info = new QLabel(i18n("NOTES"), parent);
         info->setPaletteForegroundColor(QColor(150, 0, 0));
         
         QLabel* info1 = new QLabel(i18n("Changes of audiodriver and videodriver will take effect at restart.\n\nFor immediately effect please stop playback \nbefore changing the visualization plugin."), parent);
         row += 8;
         grid->addWidget(info, row, 0);
         row ++;
         grid->addMultiCellWidget(info1, row, row, 0, 2);
       
       }        
       parent = addPage(pageName);
       parent->setFixedSize(570,400);       
       sv = new QScrollView(parent);
       sv->setResizePolicy(QScrollView::AutoOneFit);
       sv->setMinimumSize(570,400);
       parent = new QWidget(sv->viewport());
       sv->addChild(parent);
       
       grid = new QGridLayout(parent, 20 ,2);
       grid->setColStretch(1,8);
       grid->setSpacing(10);
       grid->setMargin(10);
       
       currentPage = pageName;
       row = 0;   
      }
      

    entrys.append(new XineConfigEntry(parent, grid, row, ent));
    delete ent;
    ent = new xine_cfg_entry_t;   
   }   
                         
  } while(xine_config_get_next_entry(xineEngine, ent));

 delete ent;

 show(); 
 connect(this, SIGNAL(hidden()), SLOT(delayedDestruct()));
 connect(this, SIGNAL(okClicked()), SLOT(slotOkPressed()));
 connect(this, SIGNAL(applyClicked()), SLOT(slotApplyPressed()));
  
}


XineConfig::~XineConfig()

{
 entrys.setAutoDelete(true); 
 entrys.clear();
}


void XineConfig::slotOkPressed()
{
  slotApplyPressed();
  hide();
}

void XineConfig::slotApplyPressed()
{
  xine_cfg_entry_t *entry;

  
  
  for(uint i=0; i<entrys.count(); i++)
  {
    if (entrys.at(i)->ValueChanged())
    {
      entry = new xine_cfg_entry_t;
      if (xine_config_lookup_entry(xineEngine, entrys.at(i)->GetKey(), entry))
      {
        kdDebug(555) << "Apply " << entrys.at(i)->GetKey() << "\n";
     
        entry->num_value = entrys.at(i)->GetNumValue();
        
        if (entrys.at(i)->GetStringValue()) 
          entry->str_value = (char*)entrys.at(i)->GetStringValue().latin1();

        xine_config_update_entry(xineEngine, entry);            
        delete entry;   
      }    
    }
  }    
  
}







